@@ -9,26 +9,52 @@ use Exporter();
 our @EXPORT_OK = qw(
   urandom
   urandom_ub
+  getrandom
 );
+
 our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK, );
 
-our $VERSION  = '0.38';
 our @CARP_NOT = ('Crypt::URandom');
 
-sub CRYPT_SILENT      { return 64; }               # hex 40
-sub PROV_RSA_FULL     { return 1; }
-sub VERIFY_CONTEXT    { return 4_026_531_840; }    # hex 'F0000000'
-sub W2K_MAJOR_VERSION { return 5; }
-sub W2K_MINOR_VERSION { return 0; }
-sub SINGLE_QUOTE      { return q[']; }
+BEGIN {
+    our $VERSION = '0.41';
+    eval {
+        require XSLoader;
 
-sub PATH {
+        XSLoader::load( __PACKAGE__, $VERSION );
+    } or do {
+    };
+}
+
+## no critic (ProhibitConstantPragma)
+# using constant for the speed benefit of constant-folding of values
+
+use constant CRYPT_SILENT      => 64;               # hex 40
+use constant PROV_RSA_FULL     => 1;
+use constant VERIFY_CONTEXT    => 4_026_531_840;    # hex 'F0000000'
+use constant W2K_MAJOR_VERSION => 5;
+use constant W2K_MINOR_VERSION => 0;
+use constant OS_FREEBSD => $OSNAME eq 'freebsd';
+use constant OS_WIN32 => $OSNAME eq 'MSWin32';
+use constant PATH     => do {
     my $path = '/dev/urandom';
-    if ( $OSNAME eq 'freebsd' ) {
+    if ( OS_FREEBSD() ) {
         $path = '/dev/random';    # FreeBSD's /dev/random is non-blocking
     }
-    return $path;
-}
+    $path;
+};
+use constant GETRANDOM_AVAILABLE => do {
+    my $result = 0;
+    eval {
+        my $correct_length = 2;
+        $result = getrandom($correct_length);
+    } or do {
+        $result = undef;
+    };
+    $result;
+};
+
+## use critic
 
 my $_initialised;
 my $_context;
@@ -37,7 +63,7 @@ my $_rtlgenrand;
 my $_urandom_handle;
 
 sub _init {
-    if ( $OSNAME eq 'MSWin32' ) {
+    if ( OS_WIN32() ) {
         require Win32;
         require Win32::API;
         require Win32::API::Type;
@@ -100,11 +126,8 @@ _RTLGENRANDOM_PROTO_
     else {
         require FileHandle;
         $_urandom_handle = FileHandle->new( PATH(), Fcntl::O_RDONLY() )
-          or Carp::croak( 'Failed to open '
-              . SINGLE_QUOTE()
-              . PATH()
-              . SINGLE_QUOTE()
-              . " for reading:$OS_ERROR" );
+          or Carp::croak(
+            q[Failed to open '] . PATH() . qq['. " for reading:$OS_ERROR] );
         binmode $_urandom_handle;
     }
     return;
@@ -133,11 +156,15 @@ sub _urandom {
         Carp::croak(
             'The length argument must be supplied and must be an integer');
     }
-    if ( !( ( defined $_initialised ) && ( $_initialised == $PROCESS_ID ) ) ) {
-        _init();
-        $_initialised = $PROCESS_ID;
+    if ( !GETRANDOM_AVAILABLE() ) {
+        if (
+            !( ( defined $_initialised ) && ( $_initialised == $PROCESS_ID ) ) )
+        {
+            _init();
+            $_initialised = $PROCESS_ID;
+        }
     }
-    if ( $OSNAME eq 'MSWin32' ) {
+    if ( OS_WIN32() ) {
         my $buffer = chr(0) x $length;
         if ($_cryptgenrandom) {
 
@@ -155,6 +182,9 @@ sub _urandom {
         }
         return $buffer;
     }
+    elsif ( GETRANDOM_AVAILABLE() ) {
+        return getrandom($length);
+    }
     else {
         my $result = $_urandom_handle->$type( my $buffer, $length );
         if ( defined $result ) {
@@ -164,21 +194,15 @@ sub _urandom {
             else {
                 $_urandom_handle = undef;
                 $_initialised    = undef;
-                Carp::croak( "Only read $result bytes from "
-                      . SINGLE_QUOTE()
-                      . PATH()
-                      . SINGLE_QUOTE() );
+                Carp::croak(
+                    qq[Only read $result bytes from '] . PATH() . q['] );
             }
         }
         else {
             my $error = $OS_ERROR;
             $_urandom_handle = undef;
             $_initialised    = undef;
-            Carp::croak( 'Failed to read from '
-                  . SINGLE_QUOTE()
-                  . PATH()
-                  . SINGLE_QUOTE()
-                  . ":$error" );
+            Carp::croak( q[Failed to read from '] . PATH() . qq[':$error] );
         }
     }
 }
@@ -193,7 +217,7 @@ Crypt::URandom - Provide non blocking randomness
 
 =head1 VERSION
 
-This document describes Crypt::URandom version 0.38
+This document describes Crypt::URandom version 0.41
 
 
 =head1 SYNOPSIS
@@ -213,9 +237,8 @@ OR
 This Module is intended to provide
 an interface to the strongest available source of non-blocking 
 randomness on the current platform.  Platforms currently supported are
-anything supporting /dev/urandom and versions of Windows greater than 
-or equal to Windows 2000.
-
+anything supporting L<getrandom(2)>, /dev/urandom and versions of Windows greater
+than or equal to Windows 2000.
 
 =head1 SUBROUTINES/METHODS
 
@@ -223,21 +246,33 @@ or equal to Windows 2000.
 
 =item C<urandom>
 
-=for stopwords cryptographic
+=for stopwords cryptographic Win32 initialize
 
 This function accepts an integer and returns a string of the same size
-filled with random data.  The first call will initialize the native
-cryptographic libraries (if necessary) and load all the required Perl libraries
-This call is a buffered read on non Win32 platforms.
+filled with random data. It will throw an exception if the requested amount of
+random data is not returned. The first call will initialize the native
+cryptographic libraries (if necessary) and load all the required Perl libraries.
+This call is a buffered read on non Win32 platforms that do not support L<getrandom(2)>
+or equivalent.
 
 =item C<urandom_ub>
 
-=for stopwords cryptographic
+=for stopwords cryptographic Win32 initialize unbuffered sysread
 
 This function accepts an integer and returns a string of the same size
-filled with random data.  The first call will initialize the native
-cryptographic libraries (if necessary) and load all the required Perl libraries
-This call is a unbuffered sysread on non Win32 platforms.
+filled with random data.  It will throw an exception if the requested amount of
+random data is not returned.  The first call will initialize the native
+cryptographic libraries (if necessary) and load all the required Perl libraries.
+This call is a unbuffered sysread on non Win32 platforms that do not support
+L<getrandom(2)> or equivalent.
+
+=item C<getrandom>
+
+This function accepts an integer and returns a string of the same size
+filled with random data on platforms that implement L<getrandom(2)>.
+It will throw an exception if the requested amount of random data is not returned.
+This is NOT portable across all operating systems, but is made available if
+high-speed generation of random numbers is required.
 
 =back
 
@@ -332,12 +367,7 @@ None reported.
 
 =head1 BUGS AND LIMITATIONS
 
-No bugs have been reported.
-
-Please report any bugs or feature requests to
-C<bug-crypt-urandom@rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org>.
-
+To report a bug, or view the current list of bugs, please visit L<https://github.com/david-dick/crypt-urandom/issues>
 
 =head1 AUTHOR
 
